#!/usr/bin/python2.5

import os
import re
import sys
import difflib
import optparse
import subprocess

parser = optparse.OptionParser()
parser.add_option(
    "-n",
    "--dry-run",
    action="store_true",
    dest="dry_run",
    default=False,
    help="Don't actually make any changes.",
)
parser.add_option(
    "-v",
    "--verbose",
    action="store_true",
    dest="verbose",
    default=False,
    help="Verbose output.",
)

re_vptf_sub = (
    re.compile(
        r"^from zope\.app\.pagetemplate(\.viewpagetemplatefile)?"
        r" import ViewPageTemplateFile$",
        re.M,
    ),
    r"from z3c.ptcompat import ViewPageTemplateFile",
)

re_ptf_sub = (
    re.compile(
        r"^from zope\.pagetemplate(\.pagetemplatefile)?"
        r" import PageTemplateFile$",
        re.M,
    ),
    r"from z3c.ptcompat import PageTemplateFile",
)


def log(msg):
    for line in msg.split("\n"):
        print(">>> %s" % line)


def status(msg):
    for line in msg.split("\n"):
        print("    %s" % line)


def main(options, args):
    if options.dry_run:
        status(
            "Warning: Dry run---no changes will be made to the file-system."
        )

    path = os.getcwd()
    status("Working directory: %s" % path)

    log("Analyzing source-code...")
    registry = {}
    count = 0
    for arg, dirname, names in os.walk(path):
        for name in names:
            base, ext = os.path.splitext(name)
            if ext != ".py":
                continue

            if options.verbose and count > 0 and count % 500 == 0:
                status("Processed %d files." % count)

            count += 1
            filename = os.path.join(arg, name)

            diff = create_diff(filename)
            if diff is not None:
                registry[filename] = diff

    status("%d files processed" % count)
    log("Updating import-statements in %d files..." % len(registry.keys()))

    if options.dry_run:
        queue = (["patch", "-p0", "--dry-run"],)
    else:
        queue = ["patch", "-p0", "--dry-run"], ["patch", "-p0"]

    for args in queue:
        if options.verbose:
            args.append("--verbose")
        else:
            args.append("--quiet")

        proc = subprocess.Popen(
            " ".join(args),
            shell=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )

        diffs = []
        for filename, body in registry.iteritems():
            if options.verbose:
                changes = body.count("+++ ")
                status(
                    "%s (%d %s)."
                    % (
                        filename,
                        changes,
                        "changes" if (changes > 1) else "change",
                    )
                )

            diffs.append(body)

        if not diffs:
            break

        output, err = proc.communicate("\n".join(diffs) + "\n")

        if proc.returncode != 0:
            for line in output.split("\n"):
                status(line)
            status("An error occurred while applying patches.")
            break
    else:
        status("Succesfully patched files.")

    if "--dry-run" in args:
        status("No files were changed.")

    sys.exit(proc.returncode)


def create_diff(filename):
    original = open(filename).read()
    new = original

    # process regular expression substitutions
    for exp, replacement in (re_vptf_sub, re_ptf_sub):
        new = exp.sub(replacement, new)

    if original != new:
        return ''.join(
            difflib.unified_diff(
                original.splitlines(True),
                new.splitlines(True),
                filename,
                filename)
        )


if __name__ == "__main__":
    main(*parser.parse_args())



import unittest

from zope.pagetemplate.tests import test_htmltests as reference


class HTMLTests(reference.HTMLTests):
    def setUp(self):
        import z3c.ptcompat
        import zope.component.testing
        import zope.configuration.xmlconfig

        zope.component.testing.setUp(self)
        zope.configuration.xmlconfig.XMLConfig(
            "configure.zcml", z3c.ptcompat
        )()

        super(HTMLTests, self).setUp()


class TestProgram(unittest.TestCase):
    def _makeOne(self, *args):
        from z3c.ptcompat.engine import Program

        return Program.cook(*args)

    def test_call_with_no_tal_returns_template_body(self):
        body = "<html />"
        program, _ = self._makeOne(None, body, None, None)

        result = program(None, None, tal=False)
        self.assertEqual(result, body)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED 'AS IS' AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


TESTS_REQUIRE = ["zope.configuration", "zope.testing", "zope.testrunner"]

setup(
    name="z3c.ptcompat",
    version="2.3.0",
    description="Zope-compatible page template engine based on Chameleon.",
    long_description="\n\n".join(
        (".. contents::", read("README.rst"), read("CHANGES.rst"))
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Zope Public License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Plone",
        "Framework :: Zope :: 2",
        "Framework :: Zope :: 3",
        "Framework :: Zope :: 4",
    ],
    keywords="zpt template zope",
    url="https://github.com/zopefoundation/z3c.ptcompat",
    project_urls={
        'Issue Tracker': ('https://github.com/zopefoundation/'
                          'z3c.ptcompat/issues'),
        'Sources': 'https://github.com/zopefoundation/z3c.ptcompat',
    },
    author="Zope Foundation and Contributors",
    author_email="zope-dev@zope.org",
    license="ZPL",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["z3c"],
    include_package_data=True,
    zip_safe=False,
    python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*',
    install_requires=[
        "setuptools",
        "z3c.pt >= 3.0.0a1",
        "zope.pagetemplate >= 3.6.2",
        "zope.traversing",
    ],
    extras_require={"test": TESTS_REQUIRE},
)
